﻿using Arction.Wpf.BindableCharting;
using Arction.Wpf.BindableCharting.SeriesXY;
using Arction.Wpf.BindableCharting.Views.ViewXY;
using System;
using System.ComponentModel;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace ExampleIntensityGridSeries
{
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged([System.Runtime.CompilerServices.CallerMemberName] string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        private IntensityGridSeriesCollection _ViewTopDown = new IntensityGridSeriesCollection();
        public IntensityGridSeriesCollection ViewTopDown
        {
            get { return _ViewTopDown; }
            set
            {
                _ViewTopDown = value;
                RaisePropertyChanged();
            }
        }

        private IntensityPointMatrix _Data;
        public IntensityPointMatrix Data
        {
            get { return _Data; }
            set
            {
                _Data = value;
                RaisePropertyChanged();
            }
        }

        private void button_Click(object sender, RoutedEventArgs e)
        {
            GetData();
            GetCollection();
        }

        private async void GetData()
        {
            Data = await Task.Run(() => CreateView_TopDown());
        }

        private async void GetCollection()
        {            
            ViewTopDown = new IntensityGridSeriesCollection();

            try
            {
                var data = await Task.Run(() => CreateView_TopDown());

                var plot = new IntensityGridSeries
                {
                    MouseInteraction = false,
                    Data = data.Clone() as IntensityPointMatrix,
                    AssignXAxisIndex = 0,
                    AssignYAxisIndex = 0,
                    Fill = IntensityFillStyle.Paletted,
                    ContourLineType = ContourLineTypeXY.FastColorZones,
                    ValueRangePalette = CreateGradientPalette(0, 1000)
                };

                ViewTopDown.Add(plot);
            }
            catch (Exception ex )
            {
                MessageBox.Show(ex.Message);
            }
        }

        public static IntensityPointMatrix CreateView_TopDown()
        {
            var matrix = new IntensityPointMatrix();
            var columnCount = 500;
            var segmentCount = 500;

            for (var column = 0; column < columnCount; column++)
            {
                matrix.Add(new IntensityPointCollection());
                for (var segment = 0; segment < segmentCount; segment++)
                {
                    matrix.Last().Add(new IntensityPoint(0, 0, segment + column, Colors.Blue));
                }
            }

            matrix.Freeze();
            return matrix;
        }

        public static ValueRangePalette CreateGradientPalette(double min, double max)
        {
            {
                var palette = new ValueRangePalette();

                Func<double, double> CalculateMaxValue = (double value) => (((max - min) / 100) * value) + min;

                using (var paletteStep = new PaletteStep { MaxValue = CalculateMaxValue(10), Color = (Color)ColorConverter.ConvertFromString("#495FA9") })
                    palette.Steps.Add(paletteStep);

                using (var paletteStep = new PaletteStep { MaxValue = CalculateMaxValue(40), Color = (Color)ColorConverter.ConvertFromString("#8ED1A4") })
                    palette.Steps.Add(paletteStep);

                using (var paletteStep = new PaletteStep { MaxValue = CalculateMaxValue(60), Color = (Color)ColorConverter.ConvertFromString("#F9EA75") })
                    palette.Steps.Add(paletteStep);

                using (var paletteStep = new PaletteStep { MaxValue = CalculateMaxValue(80), Color = (Color)ColorConverter.ConvertFromString("#FA9F5B") })
                    palette.Steps.Add(paletteStep);

                using (var paletteStep = new PaletteStep { MaxValue = max, Color = (Color)ColorConverter.ConvertFromString("#BE2145") })
                    palette.Steps.Add(paletteStep);

                return palette;
            }
        }
    }
}
